/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#ifndef MIRAI_ANDROID_FILE_RESOURCE_H
#define MIRAI_ANDROID_FILE_RESOURCE_H

#include "QDebug"
#include "QFile"
#include "core/BaseFileResource.h"
#include "core/TodoMd.h"

namespace mirai
{

class AndroidFileResource : public BaseFileResource
{
  public:
	AndroidFileResource(BaseFileResourceConstructor data) : BaseFileResource(data){};
	AndroidFileResource(AndroidFileResource &) = delete;
	AndroidFileResource(AndroidFileResource &&) = delete;
	AndroidFileResource operator=(AndroidFileResource &) = delete;
	AndroidFileResource operator=(AndroidFileResource &&) = delete;

	~AndroidFileResource() override = default;

	void save() override
	{
		QFile file(QString::fromStdString(getPath()));
		if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
			throw std::runtime_error("can't create " + getPath());
		}

		const std::string content = TodoMdFormat::stringifyTasks(getName(), getTasks());

		file.write(content.c_str());
		file.close();
	};

	void load() override
	{
		qDebug() << "Reading " << QString::fromStdString(getPath());
		QFile file(QString::fromStdString(getPath()));
		if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
			qDebug() << "Can't read";
			return;
		}
		QString content = file.readAll();
		file.close();

		auto result = TodoMdFormat::parse(content.toStdString());
		for (const auto &task : result.tasks) {
			addTask(task);
		}
		setName(result.name);
	};
};
} // namespace mirai
#endif
