/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#ifndef BACKEND_H
#define BACKEND_H

#include "QtCore/qcontainerfwd.h"
#include "QtCore/qtmetamacros.h"
#include "QtCore/qvariant.h"
#include "Tag.h"
#include "TasksFile.h"
#include "core/Mirai.h"
#include "core/TasksView.h"
#include <QtCore/QDateTime>
#include <QtCore/QDir>
#include <QtCore/QJsonArray>
#include <QtCore/QJsonDocument>
#include <QtCore/QJsonObject>
#include <QtCore/QList>
#include <QtCore/QObject>
#include <QtCore/QString>
#include <QtQml/qqmlregistration.h>
#include <memory>
#include <qmap.h>

#include "TaskItem.h"

class Backend : public QObject
{
	Q_OBJECT
	QML_ELEMENT
	Q_PROPERTY(QVariant tasks READ getTasks NOTIFY tasksChanged)
	Q_PROPERTY(QVariant files READ getResources NOTIFY filesChanged)
	Q_PROPERTY(QVariant tags READ getTags NOTIFY tagsChanged)
	Q_PROPERTY(QVariant activeTagsFilter READ getActiveTagsFilter NOTIFY tasksChanged)
	Q_PROPERTY(QVariant activeResourcesFilter READ getActiveResourcesFilter NOTIFY tasksChanged)
	Q_PROPERTY(bool shouldHideCompletedTasks READ shouldHideCompletedTasks WRITE hideCompletedTasks
				   NOTIFY tasksChanged)

  public:
	Backend();

	Q_INVOKABLE void addTodoFromRawFormat(QString filePath, QString text, QString date);
	Q_INVOKABLE void addTagFilter(QString tag);
	Q_INVOKABLE void removeTagFilter(QString tag);
	Q_INVOKABLE void addResourceFilter(QString fileName);
	Q_INVOKABLE void removeResourceFilter(QString fileName);
	Q_INVOKABLE void removeFilters();
	Q_INVOKABLE void updateTodoFromRawFormat(int todoIndex, QString text, QString date);
	Q_INVOKABLE void updateTodo(int todoIndex, QString state, QString text, QString date);
	Q_INVOKABLE void removeTodo(int todoIndex);
	Q_INVOKABLE void hideCompletedTasks(bool shouldHide);
	Q_INVOKABLE QString getTagColor(QString tag);
	Q_INVOKABLE void saveTagsColor(QVariant tags);
	Q_INVOKABLE void saveFilesPath(QVariant filesPath);

  private:
	void rebuildQMLTasksList();

	QVariant getTasks();
	QVariant getResources();
	QVariant getTags();
	QVariant getActiveTagsFilter();
	QVariant getActiveResourcesFilter();
	bool shouldHideCompletedTasks();
	void saveConfig();
	QString getConfigFilePath() const;

	mirai::Mirai mirai;
	// Both Todo and Calendar view use the todoView for now.
	mirai::TasksView todoView;

	QList<QMLTaskItem> QMLTasks;
	QList<QMLTasksFile> QMLResources;
	QList<QMLTag> QMLTags;
	QList<QString> QMLActiveTagsFilter;
	QList<QString> QMLActiveResourcesFilter;
	QMap<QString, QString> tagsConfig;

	bool shouldHideCompletedTasks_ = true;

  signals:

	void tasksChanged();
	void tagsChanged();
	void filesChanged();
};

#endif
