/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#include "BaseResource.h"
#include "TaskItem.h"
#include <algorithm>
#include <functional>
#include <iostream>
#include <memory>
#include <ostream>

namespace mirai
{

void BaseResource::setDirty(bool shouldBeDirty)
{
	isDirty_ = shouldBeDirty;
}

bool BaseResource::isDirty() const
{
	return isDirty_;
}

std::vector<std::unique_ptr<TaskItem>> &BaseResource::getTasks()
{
	return tasks;
}

void BaseResource::addTask(TaskItemData taskItem)
{
	tasks.push_back(std::make_unique<TaskItem>(this, taskItem));
	setDirty(true);
}

void BaseResource::removeTask(const TaskItem *taskToRemove)
{
	auto findFunction = [&](const std::unique_ptr<TaskItem> &taskInFilter) {
		return taskInFilter.get() == taskToRemove;
	};
	auto taskToDelete = std::remove_if(tasks.begin(), tasks.end(), findFunction);
	if (taskToDelete == tasks.end()) {
		return;
	}
	tasks.erase(taskToDelete);
	setDirty(true);
}

} // namespace mirai
