/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#ifndef MIRAI_TODOMD_H
#define MIRAI_TODOMD_H

#include "TaskItem.h"
#include "cpp-utils/debug.h"
#include <sstream>
#include <string>
#include <vector>

namespace mirai
{

struct MiraiMarkdownFormatParseResult {
	std::string name;
	std::vector<TaskItemData> tasks;
};

class TodoMdFormat
{
  public:
	static std::string
	stringifyTasks(const std::string &name, const std::vector<std::unique_ptr<TaskItem>> &tasks)
	{
		std::string result = "";
		std::string currentDate = "";

		result += "# " + name + "\n";
		for (const auto &task : tasks) {
			if (currentDate != task->getDate()) {
				currentDate = task->getDate();
				result += "\n## " + (task->getDate() != "" ? task->getDate() : "No date") + "\n\n";
			}
			result += taskToString(*task) + '\n';
		}
		return result;
	};

	static MiraiMarkdownFormatParseResult parse(const std::string &content)
	{
		cpputils::debug::Timer readMdFormatDuration;
		std::vector<TaskItem> taskItems;
		std::string line;

		std::stringstream contentStream(content);

		if (!std::getline(contentStream, line) || line.substr(0, 2) != "# ") {
			std::cerr << "Couldn't find the task list name" << std::endl;
		}

		std::string name = line.substr(2);
		std::string currentDate = "";

		std::vector<TaskItemData> result;

		cpputils::debug::Timer stringToTaskDuration;
		stringToTaskDuration.reset();
		cpputils::debug::Timer gelinesDuration;
		while (std::getline(contentStream, line)) {
			if (std::string_view{line.data(), 3} == "## ") {
				currentDate = line.substr(3);
			} else if (std::string_view{line.data(), 5} == "- [ ]" || std::string_view{line.data(), 5} == "- [X]") {
				stringToTaskDuration.start();
				TaskItemData taskItemData = stringToTask(line, currentDate);
				stringToTaskDuration.stop();
				taskItemData.date = currentDate;
				result.push_back(taskItemData);
			}
		}

		gelinesDuration.printTimeElapsed("getlinesDuration");
		stringToTaskDuration.printTimeElapsed("stringToTaskDuration");
		readMdFormatDuration.printTimeElapsed("Reading MD File duration");
		return {.name = name, .tasks = result};
	}

	static std::string taskToString(const TaskItem &task);
	static TaskItemData stringToTask(const std::string &str, const std::string &date);

  private:
	static std::string fieldWithSpace(const std::string &field);
	static TaskItem parseLine(const std::string &line);
	static Tags extractTagsFromMetadata(std::string metadata);
};
} // namespace mirai

#endif
